@@ -48,7 +48,7 @@ static GCallback data_from_jukebox_to_hd (GtkButton *button,
     GtkWidget *label1, *label2, *dialog, *button, *separator;
 
 #if !GTK_CHECK_VERSION(2,6,0)
-    dialog = gnome_dialog_new(_("Transfering files from jukebox file storage"),
+    dialog = gnome_dialog_new(_("Transferring files from jukebox file storage"),
 			      GNOME_STOCK_BUTTON_CANCEL,
 			      NULL);
     gnome_dialog_button_connect(GNOME_DIALOG(dialog),
@@ -56,7 +56,7 @@ static GCallback data_from_jukebox_to_hd (GtkButton *button,
 				G_CALLBACK(cancel_jukebox_operation_click),
 				NULL);
 #else
-    dialog = gtk_dialog_new_with_buttons(_("Transfering files from jukebox file storage"),
+    dialog = gtk_dialog_new_with_buttons(_("Transferring files from jukebox file storage"),
 					 NULL,
 					 0,
 					 NULL);
@@ -118,7 +118,7 @@ static GCallback data_from_hd_to_jukebox()
   if (metalist != NULL) {
     GtkWidget *label1, *label2, *dialog, *button, *separator;
 #if !GTK_CHECK_VERSION(2,6,0)
-    dialog = gnome_dialog_new(_("Transfering files to jukebox file storage"),
+    dialog = gnome_dialog_new(_("Transferring files to jukebox file storage"),
 			      GNOME_STOCK_BUTTON_CANCEL,
 			      NULL);
     gnome_dialog_button_connect(GNOME_DIALOG(dialog),
@@ -126,7 +126,7 @@ static GCallback data_from_hd_to_jukebox()
 				G_CALLBACK(cancel_jukebox_operation_click),
 				NULL);
 #else
-    dialog=gtk_dialog_new_with_buttons(_("Transfering files to jukebox file storage"),
+    dialog=gtk_dialog_new_with_buttons(_("Transferring files to jukebox file storage"),
 		    NULL,
 		    0,
 		    NULL);
