@@ -19,7 +19,9 @@ Boston, MA 02111-1307, USA.
 #include "common.h"
 /* The GNOME library - cannot be put in common as it
  * disturbs the id3.cpp file */
+#if !GTK_CHECK_VERSION(2,6,0)
 #include <libgnomeui/libgnomeui.h>
+#endif
 
 #include "filesystem.h"
 #include "jukebox.h"
@@ -311,24 +313,30 @@ static gboolean all_strings_same(GList *metalist, gint column)
 
 static void add_to_dialog(GtkWidget *dialog, GtkWidget *thing)
 {
+#if !GTK_CHECK_VERSION(2,6,0)
   gtk_box_pack_start (GTK_BOX (GNOME_DIALOG (dialog)->vbox), thing, TRUE,
 		      TRUE, 0);
+#else
+  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), thing, TRUE,
+		      TRUE, 0);
+#endif
 }
 /* Editing the metadata... */
 static void edit_dialog_create()
 {
   GtkWidget *hbox, *label, *dialog, *tempwid, *entry;
-
-  if (target == JB_LIST)
+#if !GTK_CHECK_VERSION(2,6,0)
+  if (target == JB_LIST) {
     dialog = gnome_dialog_new(_("Edit jukebox file metadata"),
 			      GNOME_STOCK_BUTTON_CANCEL,
 			      GNOME_STOCK_BUTTON_OK,
 			      NULL);
-  else
+  } else {
     dialog = gnome_dialog_new(_("Edit track metadata (ID3v2)"),
 			      GNOME_STOCK_BUTTON_CANCEL,
 			      GNOME_STOCK_BUTTON_OK,
 			      NULL);
+  }
   gnome_dialog_button_connect_object(GNOME_DIALOG(dialog),
 				     0,
 				     G_CALLBACK(edit_cancel_click),
@@ -337,6 +345,23 @@ static void edit_dialog_create()
 			      1,
 			      G_CALLBACK(edit_ok_click),
 			      NULL);
+#else
+  if (target == JB_LIST) {
+    dialog = gtk_dialog_new_with_buttons(_("Edit jukebox file metadata"),
+					 NULL, 0, NULL);
+  } else {
+    dialog = gtk_dialog_new_with_buttons(_("Edit track metadata (ID3v2)"),
+					 NULL, 0, NULL);
+  }
+  
+  tempwid = gtk_dialog_add_button(GTK_DIALOG(dialog), GTK_STOCK_CANCEL, 0);
+  g_signal_connect (tempwid, "clicked",
+		    G_CALLBACK (edit_cancel_click), NULL);
+
+  tempwid = gtk_dialog_add_button(GTK_DIALOG(dialog), GTK_STOCK_OK, 1);
+  g_signal_connect (tempwid, "clicked",
+		    G_CALLBACK (edit_ok_click), NULL);
+#endif
   edit_dialog = dialog;
 
   /* Add the metadata edit field for the dialog box */
@@ -625,7 +650,7 @@ static void edit_dialog_create()
 }
 
 /* Removes any directories from a selection */
-static GList *remove_directories(GList *metadatas)
+static GList *filter_for_editing(GList *metadatas, guint listtype)
 {
   GList *tmp = metadatas;
   GList *newmetas = NULL;
@@ -639,6 +664,11 @@ static GList *remove_directories(GList *metadatas)
     if (meta->size == 0) {
       /* Throw this away */
       destroy_metadata_t(meta);
+    } else if ( (listtype == HD_LIST) &&
+		strcmp(meta->codec, "MP3") ) {
+      // If on the host side, we can only edit MP3 ID3v1/v2
+      // metadata, so filter out anything else.
+      destroy_metadata_t(meta);
     } else {
       /* Keep this */
       newmetas = g_list_append (newmetas, meta);
@@ -654,7 +684,8 @@ static void prepare_dialog(guint listtype)
 {
   /* This is for the harddisk list only */
   GList *metadatas = get_all_metadata_from_selection(listtype);
-  metadatas = remove_directories(metadatas);
+
+  metadatas = filter_for_editing(metadatas, listtype);
   selection = get_metadata_selection(listtype);
   artist = NULL;
   title = NULL;
