@@ -101,6 +101,23 @@ const gint chomp_string_count =
 const gint replacement_string_count = 
 ((gint)(sizeof(replacement_strings)/sizeof(gchar*))-1);
 
+/* Adds an underscore before the file extension. */
+gchar *add_underscore(gchar *str)
+{
+  gchar *res;
+  gchar *res2;
+  gchar *ext;
+  guint c = strlen(str);
+  while (str[c] != '.') c--;
+  res = g_strndup(str,c);
+  ext = g_strdup(str+c);
+  res2 = g_strconcat(res, "_", ext, NULL);
+  g_free(str);
+  g_free(ext);
+  g_free(res);
+  return res2;
+}
+
 /**
  * This routine composes a suitable filename for a track file
  * based on user configuration.
