@@ -1,6 +1,6 @@
 /* filesystem.c
    Interface to the posix (well...) file system
-   Copyright (C) 2001 Linus Walleij
+   Copyright (C) 2001-2006 Linus Walleij
 
 This file is part of the GNOMAD package.
 
@@ -48,6 +48,72 @@ extern GtkWidget *main_window;
 /* Local variables */
 static gint preffd;
 
+/* 
+ * Types and function to send drawing down to idle thread
+ */
+
+typedef struct {
+  GtkWidget *label;
+  gchar *text;
+} draw_label_args_t;
+
+typedef struct {
+  GtkWidget *progress;
+  gfloat fraction;
+} fraction_set_args_t;
+
+typedef struct {
+  listtype_t listtype;
+  GList *metalist;
+} metadata_set_args_t;
+
+static gboolean draw_label(gpointer p) {
+  draw_label_args_t *args = (draw_label_args_t *) p;
+  
+  if (args != NULL) {
+    if (args->label != NULL && args->text != NULL) {
+      gtk_label_set_text(GTK_LABEL(args->label), args->text);
+    }
+    if (args->text != NULL)
+      g_free(args->text);
+    g_free(args);
+  }
+  return FALSE;
+}
+
+static gboolean update_fraction(gpointer p) {
+  fraction_set_args_t *args = (fraction_set_args_t *) p;
+  
+  if (args != NULL) {
+    gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(args->progress), args->fraction);
+    g_free(args);
+  }
+  return FALSE;
+}
+
+static gboolean update_metadata(gpointer p) {
+  metadata_set_args_t *metarg = (metadata_set_args_t *) p;
+  GList *tmp;
+
+  if (metarg != NULL) {
+    recreate_list_store(metarg->listtype);
+    tmp = metarg->metalist;
+    while (tmp) {
+      metadata_t *meta;
+      
+      meta = (metadata_t *) tmp->data;
+      // g_print("Adding metadata for \"%s\" to model\n", meta->path);
+      add_metadata_to_model(meta, metarg->listtype);
+      tmp = tmp->next;
+      destroy_metadata_t(meta);
+    }
+    g_list_free(metarg->metalist);
+    view_and_sort_list_store(metarg->listtype);
+    g_free(metarg);
+  }
+  return FALSE;
+}
+
 /* Filecopy routine loosely based on
  * http://www.snippets.org/snippets/portable/WB_FCOPY+C.php3 */
 gboolean copy_file(gchar *source, gchar *dest)
@@ -214,7 +280,7 @@ static GList *get_directory_listing(gchar* base_path, gboolean anyfile)
       strncpy(path_buffer, tmppath, 512);
       strcat(path_buffer, G_DIR_SEPARATOR_S);
       strcat(path_buffer, dir_entry->d_name);
-      if (stat(path_buffer, &entry_status)) {
+      if (g_stat(path_buffer, &entry_status)) {
 	if (gnomad_debug != 0) {
 	  g_print("Failed to read file status for %s.\n", path_buffer);
 	}
@@ -260,6 +326,10 @@ static GList *get_directory_listing(gchar* base_path, gboolean anyfile)
 	      // Additional music codecs supported by libmtp
 	      &&
 	      g_ascii_strcasecmp(tmp,"ogg") &&
+	      g_ascii_strcasecmp(tmp,"flac") &&
+	      g_ascii_strcasecmp(tmp,"mp2") &&
+	      g_ascii_strcasecmp(tmp,"m4a") &&
+	      g_ascii_strcasecmp(tmp,"aac") &&
 	      g_ascii_strcasecmp(tmp,"mp4")
 #endif
 	      ) {
@@ -326,7 +396,7 @@ static GList *get_metadata_for_dirlist(GList *inlist,
     guint size;
     gint i;
     gchar fileext[5];
-    
+        
     meta = new_metadata_t();
     /* Full path -> ID column */
     path = filelist->data;
@@ -347,16 +417,20 @@ static GList *get_metadata_for_dirlist(GList *inlist,
       g_free(tmp);
     }
     if (use_dialog) {
-      gdk_threads_enter();
-      tmp = g_strdup_printf("File %d of %d", currentfile, nofiles);
+      draw_label_args_t *da;
+
+      da = (draw_label_args_t *) g_malloc(sizeof(draw_label_args_t));
+      da->label = args->label;
+      da->text = g_strdup_printf("File %d of %d", currentfile, nofiles);
+      g_idle_add(draw_label, da);
+
+      currentfile ++;
       if (gnomad_debug != 0) {
 	g_print("Getting metadata for: %s (%d/%d)\n", meta->filename, currentfile, nofiles);
       }
-      currentfile ++;
-      gtk_label_set_text(GTK_LABEL(args->label), tmp);
-      gdk_threads_leave();
     }
     /* Find codec/file extension first */
+    fileext[0] = 0;
     if (strlen(meta->path) > 5) {
       gchar *tmp = meta->path;
 
@@ -404,9 +478,18 @@ static GList *get_metadata_for_dirlist(GList *inlist,
 	  get_tag_for_wmafile(meta);
 	}
 #ifdef HAVE_LIBMTP
-	// Implement getting track metadata for ogg and mp4 here.
+	// Implement getting track metadata for the other libmtp
+	// supported filetypes here...
 	if (!strcmp(meta->codec,"OGG")) {
 	}
+	if (!strcmp(meta->codec,"FLAC")) {
+	}
+	if (!strcmp(meta->codec,"MP2")) {
+	}
+	if (!strcmp(meta->codec,"M4A")) {
+	}
+	if (!strcmp(meta->codec,"AAC")) {
+	}
 	if (!strcmp(meta->codec,"MP4")) {
 	}
 #endif
@@ -461,16 +544,21 @@ static GList *get_metadata_for_dirlist(GList *inlist,
     metalist = g_list_append(metalist, meta);
     /* Update progress bar */
     if (use_dialog) {
+      fraction_set_args_t *fsa;
+
       gfloat fraction = ((gdouble) currentfile / (gdouble) nofiles);
-      gdk_threads_enter();
-      gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(args->progress), fraction);
-      gdk_threads_leave();
+
+      fsa = (fraction_set_args_t *) g_malloc(sizeof(fraction_set_args_t));
+      fsa->progress = args->progress;
+      fsa->fraction = fraction;
+      g_idle_add(update_fraction, fsa);
       cancelled = args->cancelled;
     }
   }
   return metalist;
 }
 
+
 /**
  * @param args if this and the following parameter is null the call is assumed to be 
  *        non-threaded, else it is updated and handled as if called from a thread.
@@ -487,9 +575,12 @@ GList *get_metadata_dir(listtype_t listtype,
 
   /* If it is for the data view, get for ALL files */
   if (args != NULL) {
-    gdk_threads_enter();
-    gtk_label_set_text(GTK_LABEL(args->label), _("Getting file list from filesystem"));
-    gdk_threads_leave();
+    draw_label_args_t *da;
+
+    da = (draw_label_args_t *) g_malloc(sizeof(draw_label_args_t));
+    da->label = args->label;
+    da->text = g_strdup(_("Getting file list from filesystem"));
+    g_idle_add(draw_label, da);
   }
   filelist = get_directory_listing(path, (listtype == HDDATA_LIST));
   /* This is the heavy part */
@@ -509,28 +600,23 @@ gpointer dirfill_thread(gpointer thread_args)
 {
   dirfill_thread_arg_t *args = (dirfill_thread_arg_t *) thread_args;
   GList *metalist;
-  GList *tmp;
+  draw_label_args_t *da;
+  metadata_set_args_t *metarg;
 
   /* This is time consuming */
   metalist = get_metadata_dir(args->listtype, args->path, args);
 
-  /* This will interfere with other threads. */
-  gdk_threads_enter();
-  gtk_label_set_text(GTK_LABEL(args->label), _("Adding metadata to view"));
-  recreate_list_store(args->listtype);
-  tmp = metalist;
-  while (tmp) {
-    metadata_t *meta;
-    
-    meta = (metadata_t *) tmp->data;
-    add_metadata_to_model(meta, args->listtype);
-    tmp = tmp->next;
-    destroy_metadata_t(meta);
-  }
-  view_and_sort_list_store(args->listtype);
-  gtk_widget_destroy(args->dialog);
-  gdk_threads_leave();
-  g_list_free(metalist);
+  da = (draw_label_args_t *) g_malloc(sizeof(draw_label_args_t));
+  da->label = args->label;
+  da->text = g_strdup(_("Adding metadata to view"));
+  g_idle_add(draw_label, da);  
+
+  metarg = (metadata_set_args_t *) g_malloc(sizeof(metadata_set_args_t));
+  metarg->listtype = args->listtype;
+  metarg->metalist = metalist;
+  g_idle_add(update_metadata,metarg);
+
+  g_idle_add((GSourceFunc) gtk_widget_destroy, args->dialog);
   g_free(args->path);
   g_free(args);
 }
@@ -570,10 +656,6 @@ void fill_in_dir(listtype_t listtype, gchar *path)
   dirfill_thread_arg_t *dirfill_thread_args;
   GtkWidget *label1;
   GtkWidget *dialog, *scanlabel, *metadata_progress_bar, *separator;
-#ifdef G_OS_WIN32
-  GList *metalist = NULL;
-  GList *tmp;
-#endif
 
   dirfill_thread_args = (dirfill_thread_arg_t *) g_malloc(sizeof(dirfill_thread_arg_t));
 
@@ -639,31 +721,7 @@ void fill_in_dir(listtype_t listtype, gchar *path)
   dirfill_thread_args->path = g_strdup(path);
   dirfill_thread_args->cancelled = FALSE;
 
-#ifndef G_OS_WIN32
-  // TODO: check no reentrance while this thread is running...
   g_thread_create(dirfill_thread,(gpointer) dirfill_thread_args, FALSE, NULL);
-#else
-  // Windows cannot do all things in the threads. Therefore we just unthread
-  // it. This is a bit time consuming...
-  metalist = get_metadata_dir(dirfill_thread_args->listtype, dirfill_thread_args->path, NULL);
-
-  gtk_label_set_text(GTK_LABEL(dirfill_thread_args->label), _("Adding metadata to view"));
-  recreate_list_store(dirfill_thread_args->listtype);
-  tmp = metalist;
-  while (tmp) {
-    metadata_t *meta;
-    
-    meta = (metadata_t *) tmp->data;
-    add_metadata_to_model(meta, dirfill_thread_args->listtype);
-    tmp = tmp->next;
-    destroy_metadata_t(meta);
-  }
-  view_and_sort_list_store(dirfill_thread_args->listtype);
-  gtk_widget_destroy(dirfill_thread_args->dialog);
-  g_list_free(metalist);
-  g_free(dirfill_thread_args->path);
-  g_free(dirfill_thread_args);
-#endif
 }
 
 gchar *get_current_dir()
@@ -690,23 +748,59 @@ void change_directory(gchar *dir)
   g_free(tmp);
 }
 
-void delete_files(GList *filelist)
+gint remove_dir_contents(gchar *dir)
+{
+  GDir * current_dir;
+  gchar * file = NULL;
+  gchar * current_file = NULL;
+  gint c = 0;
+
+  if ((current_dir = g_dir_open(dir,0,NULL)) == NULL) return -1;
+
+  while ((file = g_dir_read_name(current_dir)) != NULL)
+    {
+      current_file = g_build_filename (dir, file, NULL);
+
+      if (g_file_test(current_file, G_FILE_TEST_IS_DIR))
+	{
+	  c += remove_dir_contents(current_file);
+	}
+      
+#if !GLIB_CHECK_VERSION(2,6,0)
+      c += remove(current_file);
+#else
+      c += g_remove(current_file);
+#endif
+      g_free(current_file);
+    }
+
+  g_dir_close(current_dir);
+  return c;
+}
+
+gint delete_files(GList *filelist)
 {
   GList *tmplist = g_list_first(filelist);
   gchar *tmp;
+  guint c = 0;
 
   while (tmplist != NULL) {
     metadata_t *meta = (metadata_t *) tmplist->data;
     /* g_print("Deleting: %s\n", meta->path); */
     tmp = filename_fromutf8(meta->path);
+
+    if (g_file_test(tmp, G_FILE_TEST_IS_DIR)) remove_dir_contents(tmp);
+
 #if !GLIB_CHECK_VERSION(2,6,0)
-    unlink(tmp);
+    c += remove(tmp);
 #else
-    g_unlink(tmp);
+    c += g_remove(tmp);
 #endif
+
     g_free(tmp);
     tmplist = tmplist->next;
   }
+  return c;
 }
 
 /* Copy an MP3 file and remove the tag */
