@@ -26,6 +26,11 @@ Boston, MA 02111-1307, USA.
 #if !GTK_CHECK_VERSION(2,6,0)
 #include <libgnomeui/libgnomeui.h>
 #endif
+#ifdef HAVE_DBUSGLIB
+#define DBUS_API_SUBJECT_TO_CHANGE
+#include <dbus/dbus-glib.h>
+#include <dbus/dbus.h>
+#endif
 
 /* This one should be global really */
 GtkWidget *main_window;
@@ -42,6 +47,11 @@ static GtkWidget *jukebox_menu;
 #endif
 // The expose() event will be treated JUST ONCE at the first window creation
 static gboolean justonce = TRUE;
+#ifdef HAVE_DBUSGLIB
+DBusConnection *dbus_conn;
+DBusError dbus_error;
+#endif
+
 
 /* Expect these functions */
 void scan_jukebox(gpointer data);
@@ -100,6 +110,9 @@ static GCallback jukebox_ok_signal(GtkButton *button,
   if (jukebox_select(selection)) {
     // Initially scan the jukebox
     scan_jukebox(NULL);
+#ifdef HAVE_DBUSGLIB
+    // Transmit jukebox connection message on D-BUS
+#endif
   }
 }
 #endif
@@ -181,6 +194,9 @@ static void create_select_jukebox_dialog(void)
       if (jukebox_select(0)) {
 	// Initially scan the jukebox
 	scan_jukebox(NULL);
+#ifdef HAVE_DBUSGLIB
+    // Transmit jukebox connection message on D-BUS
+#endif
       }
     }
   }
@@ -209,12 +225,14 @@ static gint expose(GtkWidget *widget,
 /* The about box */
 static void about_box(gpointer data)
 {
-  const gchar *authors[] = {"Linus Walleij - main code, parts of libnjb", 
+  const gchar *authors[] = {"Linus Walleij - main code, parts of libnjb and libmtp", 
 			    "John Mechalas (Seagull) - libnjb",
-			    "César Talón - Jukebox 3 examinations", NULL};
-  const gchar *copystring = "Copyright (C) 2002 Linus Walleij";
+			    "César Talón - Jukebox 3 examinations", 
+			    "Richard A. Low - initial MTP work and libmtp",
+			    NULL};
+  const gchar *copystring = "Copyright (C) 2006 Linus Walleij";
   const gchar *thanks[] = {"ALL YOUR BASE ARE", "BELONG TO US", NULL};
-  gchar *aboutstring = _("A program to communicate with\nCreative Jukeboxes");
+  gchar *aboutstring = _("A program to communicate with\nCreative Jukeboxes\nand MTP devices");
   gchar *translator_credits = _("translator_credits");
 
   GtkWidget *about;
@@ -466,7 +484,6 @@ int main(int argc,
   textdomain (GETTEXT_PACKAGE);
 
   g_thread_init(NULL);
-  gdk_threads_init();
 
 #if GTK_CHECK_VERSION(2,6,0)
   gtk_init(&argc, &argv);
@@ -656,11 +673,31 @@ int main(int argc,
   gtk_menu_item_set_right_justified(GTK_MENU_ITEM(root_menu), TRUE);
   gtk_menu_item_set_submenu(GTK_MENU_ITEM(root_menu), menu);
   gtk_menu_shell_append(GTK_MENU_SHELL(menu_bar), root_menu);
+
+#ifdef HAVE_DBUSGLIB
+  dbus_error_init(&dbus_error);
+
+  // Open D-BUS connection
+  dbus_conn = dbus_bus_get(DBUS_BUS_SESSION, &dbus_error);
+  if (dbus_conn == NULL) {
+    g_printerr ("Failed to open connection to D-BUS bus\n");
+  } else {
+    DBusMessage *message;
+
+    dbus_connection_setup_with_g_main(dbus_conn, NULL);
+    message = dbus_message_new_signal ("/net/sourceforge/gnomad2",
+				       "net.sourceforge.gnomad2", 
+				       "DeviceStatus");
+    // Append the string "Online" to the signal
+    //dbus_message_append_args(message,DBUS_TYPE_STRING,"Online",DBUS_TYPE_INVALID);
+    dbus_connection_send(dbus_conn, message, NULL);
+    dbus_message_unref(message);
+  }
+  dbus_error_free(&dbus_error);
+#endif
   
   gtk_widget_show(main_window);
 
-  gdk_threads_enter();
   gtk_main();
-  gdk_threads_leave();
   gtk_exit(0);
 }
