@@ -703,9 +703,12 @@ remove_or_add_tag(gboolean add, gchar *path, guchar *v2tag, guint v2taglen, guch
 	g_print(" %d (0x%x) bytes\n", header_taglength, header_taglength);
       }
 
+      // Wind past ID3 header
+      lseek(f1, 10+header_taglength, SEEK_SET);
+      
       // Wind past any RIFF header too
       n = read(f1,buffer,bufsiz);
-      header_taglength += riff_header_size(&buffer[header_taglength], n);
+      header_taglength += riff_header_size(&buffer[0], n);
       if (gnomad_debug != 0) {
 	g_print("ID3v2 header (and any RIFF header) %d (0x%x) bytes\n", header_taglength, header_taglength);
       }
