@@ -2,7 +2,7 @@
  * \file jukebox.c
  *  Functions related to the communication with jukebox hardware
  *  using the libnjb library.
- *  Copyright (C) 2001-2003 Linus Walleij
+ *  Copyright (C) 2001-2007 Linus Walleij
  *
  * This file is part of the GNOMAD package.
  *
@@ -119,11 +119,25 @@ static const mtp_filetype_description_t mtp_filetype_descriptions[] = {
   { LIBMTP_FILETYPE_PNG,     "PNG" },
   { LIBMTP_FILETYPE_WINDOWSIMAGEFORMAT, "WMF" },
   { LIBMTP_FILETYPE_VCALENDAR2, "ICS" },
+  { LIBMTP_FILETYPE_VCARD3,  "VCF" },
   { LIBMTP_FILETYPE_WINEXEC, "EXE" },
   { LIBMTP_FILETYPE_WINEXEC, "COM" },
   { LIBMTP_FILETYPE_WINEXEC, "BAT" },
   { LIBMTP_FILETYPE_WINEXEC, "DLL" },
   { LIBMTP_FILETYPE_WINEXEC, "SYS" },
+  { LIBMTP_FILETYPE_TEXT,    "TXT" },
+  { LIBMTP_FILETYPE_HTML,    "HTML" },
+  { LIBMTP_FILETYPE_AAC,     "AAC" },
+  { LIBMTP_FILETYPE_FLAC,    "FLAC" },
+  { LIBMTP_FILETYPE_MP2,     "MP2" },
+  { LIBMTP_FILETYPE_M4A,     "M4A" },
+  { LIBMTP_FILETYPE_DOC,     "DOC" },
+  { LIBMTP_FILETYPE_XML,     "XML" },
+  { LIBMTP_FILETYPE_XLS,     "XLS" },
+  { LIBMTP_FILETYPE_PPT,     "PPT" },
+  { LIBMTP_FILETYPE_MHT,     "MHT" },
+  { LIBMTP_FILETYPE_JP2,     "JP2" },
+  { LIBMTP_FILETYPE_JPX,     "JPX" },
   { LIBMTP_FILETYPE_UNKNOWN, "N/A" }
 };
 
@@ -446,7 +460,7 @@ GtkWidget *jukebox_get_deviceinfo_dialog(void)
   add_to_dialog(dialog, hbox);
   gtk_widget_show (hbox);
 
-  if (use_mtp || (pdedevice->device_type != NJB_DEVICE_NJB1)) {
+  if (!use_mtp && (pdedevice->device_type != NJB_DEVICE_NJB1)) {
     hbox = gtk_hbox_new(FALSE, 0);
     label = gtk_label_new (_("Hardware revision:"));
     gtk_box_pack_start (GTK_BOX(hbox), label, FALSE, FALSE, 0);
@@ -675,11 +689,8 @@ static void add_row_from_songhash (gpointer key,
 }
 
 
-static void jblist_from_songhash(gboolean threaded)
+static void jblist_from_songhash(void)
 {
-  if (threaded)
-    gdk_threads_enter();
-
   /* Little trick I learned from Sven Neumann, recreate the store each time,
    * because otherwise you cannot remove the sorting attribute from the store,
    * and that in turn will result in O(n^2) addition times for this store.
@@ -693,8 +704,6 @@ static void jblist_from_songhash(gboolean threaded)
 		       NULL);
   /* Then sort everything */
   view_and_sort_list_store(JB_LIST);
-  if (threaded)
-    gdk_threads_leave();
 }
 
 
@@ -1024,12 +1033,10 @@ static void create_tree_structure()
 }
 
 
-static void build_tree_widget(GtkTreeStore *pltreestore, gboolean threaded)
+static void build_tree_widget(GtkTreeStore *pltreestore)
 {
   GSList *tmplist;
 
-  if (threaded)
-    gdk_threads_enter();
   clear_list_store(PL_TREE);
   tmplist = playlistlist;
   // Draw the tree...
@@ -1060,6 +1067,8 @@ static void build_tree_widget(GtkTreeStore *pltreestore, gboolean threaded)
        * in their playlists... Fixed 2004-03-09 Linus Walleij
        */
       if (meta != NULL) {
+	// g_print("Adding song with ID %lu to playlist %lu\n", GPOINTER_TO_UINT(tmplist2->data), entry->plid);
+	// dump_metadata_t(meta);
 	gtk_tree_store_append (GTK_TREE_STORE(pltreestore), &treeiter2, &treeiter);
 	gtk_tree_store_set(GTK_TREE_STORE(pltreestore), &treeiter2,
 			   PLIST_ARTIST_COLUMN, meta->artist,
@@ -1075,9 +1084,6 @@ static void build_tree_widget(GtkTreeStore *pltreestore, gboolean threaded)
     }
     tmplist = tmplist->next;
   }
-  if (threaded)
-    gdk_threads_leave();  
-  
 }
 
 static void destroy_tree_structure()
@@ -1100,9 +1106,9 @@ static void destroy_tree_structure()
 }
 
 
-static void build_playlist_tree(GtkTreeStore *pltreestore,
-				gboolean threaded)
+void build_playlist_tree(GtkTreeStore *pltreestore)
 {
+  jukebox_locked = TRUE;
   /* First build the structure for dialog boxes */
   build_playlist_list();
   /* Next get the playlists and their respective tracks
@@ -1111,114 +1117,23 @@ static void build_playlist_tree(GtkTreeStore *pltreestore,
   /* Now we are finished with the jukebox communication and we have
    * all playlists in memory, so let's enter the GTK thread and
    * redraw the tree. */
-  build_tree_widget(pltreestore, threaded);
+  build_tree_widget(pltreestore);
   /* Destroy the memory allocated by temporary playlist
    * tree structure */
   destroy_tree_structure();
+  jukebox_locked = FALSE;
 }
 
-static void add_to_playlist(guint playlistid, GList *metalist, 
-			    GtkTreeStore *pltreestore, gboolean threaded)
+/*
+ * This function is used to push the playlist tree building
+ * down to the main loop.
+ */
+static gboolean call_build_playlist_tree(gpointer data)
 {
-  // Sanity check
-  if (!metalist)
-    return;
+  GtkTreeStore *pltreestore = (GtkTreeStore *) data;
 
-  if (use_mtp) {
-#ifdef HAVE_LIBMTP
-    LIBMTP_playlist_t *pl;
-    GList *tmplist;
-    guint i;
-    guint toadd;
-    uint32_t *copy;
-    
-    pl = LIBMTP_Get_Playlist(mtpdevice, playlistid);
-    if (pl == NULL) {
-      return;
-    }
-
-    // Count the tracks and allocate space for playlist
-    tmplist = metalist;
-    toadd = 0;
-    while(tmplist) {
-      toadd++;
-      tmplist = g_list_next(tmplist);
-    }
-    
-    // Allocate the new playlist
-    copy = g_malloc((pl->no_tracks + toadd) * sizeof(uint32_t));
-    
-    // Copy old tracks
-    for (i = 0; i < pl->no_tracks; i++) {
-      copy[i] = pl->tracks[i];
-    }
-
-    // Remove old contents
-    if (pl->tracks != NULL) {
-      g_free(pl->tracks);
-    }
-    pl->tracks = copy;
-    pl->no_tracks = pl->no_tracks + toadd;
-
-    // Add each new track
-    tmplist = metalist;
-    while(tmplist) {
-      metadata_t *meta;
-      uint32_t id;
-      
-      meta = (metadata_t *) tmplist->data;
-      id = string_to_guint(meta->path);
-      /* g_print("Adding track %lu to playlist %lu\n", id, playlistid); */
-      pl->tracks[i] = id;
-      tmplist = g_list_next(tmplist);
-      i++;
-    }
-    if (LIBMTP_Update_Playlist(mtpdevice, pl) != 0) {
-      g_print("Failed to add tracks to playlist on MTP device.\n");
-    }
-#endif
-  } else {
-    njb_playlist_track_t *pl_track;
-    njb_playlist_t *playlist;
-    GList *tmplist = metalist;
-    gboolean found = FALSE;
-    
-    NJB_Reset_Get_Playlist(pdedevice);
-    while (playlist = NJB_Get_Playlist(pdedevice)) {
-      /* g_print("Is it %lu?\n", playlist->plid); */
-      if (playlist->plid == playlistid) {
-	/* g_print("YES!\n"); */
-	found = TRUE;
-	break;
-      }
-      /* g_print("NO.\n"); */
-      NJB_Playlist_Destroy(playlist);
-    }
-    if (!found) {
-      if (threaded)
-	gdk_threads_enter();
-      create_error_dialog(_("Could not find playlist"));
-      if (threaded)
-	gdk_threads_leave();
-    } else if (tmplist) {
-      while(tmplist) {
-	metadata_t *meta;
-	u_int32_t id;
-	
-	meta = (metadata_t *) tmplist->data;
-	id = string_to_guint(meta->path);
-	/* g_print("Adding track %lu to playlist %lu\n", id, playlistid); */
-	pl_track = NJB_Playlist_Track_New(id);
-	NJB_Playlist_Addtrack(playlist, pl_track, NJB_PL_END);
-	tmplist = g_list_next(tmplist);
-      }
-      if (NJB_Update_Playlist(pdedevice, playlist) == -1)
-	NJB_Error_Dump(pdedevice, stderr);
-      NJB_Playlist_Destroy(playlist);
-    }
-  }
-  /* Rebuild the tree afterwards */
-  build_playlist_tree(pltreestore, threaded);
+  build_playlist_tree(pltreestore);
+  return FALSE;
 }
 
 /* Update the usage information */
@@ -1230,21 +1145,16 @@ static void flush_usage()
   if (use_mtp) {
 #ifdef HAVE_LIBMTP
     gint ret;
-    gchar *desc;
-    gchar *label;
 
-    ret = LIBMTP_Get_Storageinfo(mtpdevice, &totalbytes, &freebytes, &desc, &label);
+    ret = LIBMTP_Get_Storage(mtpdevice, 0);
+    if (mtpdevice->storage != NULL) {
+      totalbytes = mtpdevice->storage->MaxCapacity;
+      freebytes = mtpdevice->storage->FreeSpaceInBytes;
+    }
     if (ret != 0) {
       totalbytes = 0;
       freebytes = 0;
     }
-    // These are not used right now.
-    if (desc != NULL) {
-      g_free(desc);
-    }
-    if (label != NULL) {
-      g_free(label);
-    }
 #endif
   } else {
     if (NJB_Get_Disk_Usage (pdedevice, &totalbytes, &freebytes) == -1) {
@@ -1394,19 +1304,58 @@ void rebuild_datafile_list(gchar *filter) {
   }
 }
 
-#ifdef G_OS_WIN32
-// Since windows cannot thread as it should, this function
-// will be called from the main look instead.
-static gboolean call_build_all(gpointer data)
-{
-  GtkTreeStore *pltreestore = (GtkTreeStore *) data;  
+/*
+ * These structs and functions are only here to be able to
+ * push all drawing operations up to the main GTK (idle) loop.
+ */
 
-  jblist_from_songhash(FALSE);
+typedef struct {
+  GtkWidget *label;
+  gchar *text;
+} draw_label_args_t;
+
+typedef struct {
+  GtkWidget *progress;
+  gfloat fraction;
+} draw_progress_args_t;
+
+static gboolean draw_label(gpointer p) {
+  draw_label_args_t *args = (draw_label_args_t *) p;
+  
+  if (args != NULL) {
+    if (args->label != NULL && args->text != NULL) {
+      gtk_label_set_text(GTK_LABEL(args->label), args->text);
+    }
+    if (args->text != NULL)
+      g_free(args->text);
+    g_free(args);
+  }
+  return FALSE;
+}
+
+static gboolean draw_progress(gpointer p) {
+  draw_progress_args_t *args = (draw_progress_args_t *) p;
+  
+  if (args != NULL) {
+    if (args->progress != NULL) {
+      gtk_progress_bar_update(GTK_PROGRESS_BAR(args->progress), args->fraction);
+    }
+    g_free(args);
+  }
+  return FALSE;
+}
+
+static gboolean call_jblist_from_songhash(gpointer data)
+{
+  jblist_from_songhash();
+  return FALSE;
+}
+
+static gboolean call_rebuild_datafile_list(gpointer data)
+{
   rebuild_datafile_list("\\");
-  build_playlist_tree(pltreestore, FALSE);
   return FALSE;
 }
-#endif
 
 /**
  * Scanning thread - reads in the Jukebox track directory, the datafile directory
@@ -1426,12 +1375,7 @@ gpointer scan_thread(gpointer thread_args)
   LIBMTP_file_t *mtpfiles;
   LIBMTP_file_t *mtpfile;
 #endif
-#ifndef G_OS_WIN32
-  gchar *scan_songs = _("Scanning songs...");
-  gchar *slow_warning = _("Scanning songs:\nYou have selected extended metadata scan,\n so scanning will be very slow.");
-  gchar *scan_playlists = _("Scanning playlists...");
-  gchar *scan_datafiles = _("Scanning datafiles...");
-#endif
+  draw_label_args_t *da;
   scan_thread_arg_t *args = (scan_thread_arg_t *) thread_args;
 
   /* Destroy old song hashes */
@@ -1441,15 +1385,15 @@ gpointer scan_thread(gpointer thread_args)
   jukebox_playlists = 0;
 
   /* Retrieve general information about the jukebox */
-#ifndef G_OS_WIN32
-  gdk_threads_enter();
+  da = (draw_label_args_t *) g_malloc(sizeof(draw_label_args_t));
+  da->label = args->label;
   if (get_prefs_extended_metadata()) {
-    gtk_label_set_text(GTK_LABEL(args->label), slow_warning);
+    da->text = g_strdup(_("Scanning songs:\nYou have selected extended metadata scan,\n so scanning will be very slow."));
   } else {
-    gtk_label_set_text(GTK_LABEL(args->label), scan_songs);
+    da->text = g_strdup(_("Scanning songs..."));
   }
-  gdk_threads_leave();
-#endif
+  g_idle_add(draw_label, da);
+
   /* If it has already been read, free the old string */
   if (jukebox_ownerstring != NULL)
     g_free(jukebox_ownerstring);
@@ -1478,7 +1422,7 @@ gpointer scan_thread(gpointer thread_args)
   /* Next retrieve the track listing */
   if (use_mtp) {
 #ifdef HAVE_LIBMTP
-    mtptracks = LIBMTP_Get_Tracklisting(mtpdevice);
+    mtptracks = LIBMTP_Get_Tracklisting_With_Callback(mtpdevice,NULL,NULL);
     mtptrack = mtptracks;
     if (mtptrack == NULL) {
       last_item = TRUE;
@@ -1503,17 +1447,14 @@ gpointer scan_thread(gpointer thread_args)
   while (!last_item) {
     gint i;
     metadata_t *meta;
-    gchar *scanstring;
 
     jukebox_songs++;
     /* Number of songs scanned during scan. */
-#ifndef G_OS_WIN32
-    scanstring = g_strdup_printf(_("%u songs scanned"), jukebox_songs);
-    gdk_threads_enter();
-    gtk_label_set_text(GTK_LABEL(args->label), scanstring);
-    gdk_threads_leave();
-    g_free(scanstring);
-#endif
+    da = (draw_label_args_t *) g_malloc(sizeof(draw_label_args_t));
+    da->label = args->label;
+    da->text = g_strdup_printf(_("%u songs scanned"), jukebox_songs);
+    g_idle_add(draw_label, da);
+    
     // Create a structure to hold the data in the columns
     meta = new_metadata_t();
 
@@ -1630,24 +1571,18 @@ gpointer scan_thread(gpointer thread_args)
   }
 
   /* Build songlist from hash */
-#ifndef G_OS_WIN32
-  if (gnomad_debug != 0) {
-    g_print("Building songlist from hash...\n");
-  }
-  jblist_from_songhash(TRUE);
-#endif
-
-#ifndef G_OS_WIN32
-  gdk_threads_enter();
-  gtk_label_set_text(GTK_LABEL(args->label), scan_datafiles);
-  gdk_threads_leave();
-#endif
+  g_idle_add(call_jblist_from_songhash, NULL);
 
+  da = (draw_label_args_t *) g_malloc(sizeof(draw_label_args_t));
+  da->label = args->label;
+  da->text = g_strdup(_("Scanning datafiles..."));
+  g_idle_add(draw_label, da);
+  
   /* Next retrieve the datafile listing */
   last_item = FALSE;
   if (use_mtp) {
 #ifdef HAVE_LIBMTP
-    mtpfiles = LIBMTP_Get_Filelisting(mtpdevice);
+    mtpfiles = LIBMTP_Get_Filelisting_With_Callback(mtpdevice,NULL,NULL);
     mtpfile = mtpfiles;
     if (mtpfile == NULL) {
       last_item = TRUE;
@@ -1666,7 +1601,6 @@ gpointer scan_thread(gpointer thread_args)
   /* Destroy the old datafile list */
   destroy_datafile_list();
 
-
   while (!last_item) {
     metadata_t *meta;
     u_int64_t filesize;
@@ -1675,13 +1609,10 @@ gpointer scan_thread(gpointer thread_args)
     
     jukebox_datafiles++;
 
-#ifndef G_OS_WIN32
-    scanstring = g_strdup_printf(_("%u data files scanned"), jukebox_datafiles);
-    gdk_threads_enter();
-    gtk_label_set_text(GTK_LABEL(args->label), scanstring);
-    gdk_threads_leave();
-    g_free(scanstring);
-#endif
+    da = (draw_label_args_t *) g_malloc(sizeof(draw_label_args_t));
+    da->label = args->label;
+    da->text = g_strdup_printf(_("%u data files scanned"), jukebox_datafiles);
+    g_idle_add(draw_label, da);
 
     // Create a structure to hold the data in the columns
     meta = new_metadata_t();
@@ -1742,40 +1673,33 @@ gpointer scan_thread(gpointer thread_args)
   }
   
   /* Then refill and sort everything */
-#ifndef G_OS_WIN32
-  gdk_threads_enter();
-  rebuild_datafile_list("\\");
-  gtk_label_set_text(GTK_LABEL(args->label), scan_playlists);
-  gdk_threads_leave();
-#endif
+  da = (draw_label_args_t *) g_malloc(sizeof(draw_label_args_t));
+  da->label = args->label;
+  da->text = g_strdup(_("Scanning playlists..."));
+  g_idle_add(draw_label, da);
+
+  /* Tell main loop to rebuild datafile list */
+  g_idle_add(call_rebuild_datafile_list, NULL);
   
   // Then refresh the playlist tree
   if (gnomad_debug != 0) {
     g_print("Building playlist tree...\n");
   }
-#ifndef G_OS_WIN32
-  build_playlist_tree(args->pltreestore, TRUE);
-  gdk_threads_enter();
-  gtk_widget_destroy(args->dialog);
-  gdk_threads_leave();
-#else
-  // This will call all widget rebuilding from within the main
-  // loop instead on Windows.
-  g_idle_add(call_build_all, (gpointer) args->pltreestore);
-  gtk_widget_destroy(args->dialog);
-#endif
+  g_idle_add(call_build_playlist_tree, (gpointer) args->pltreestore);
+  g_idle_add((GSourceFunc) gtk_widget_destroy, args->dialog);
   
   jukebox_locked = FALSE;
 }
 
 static set_progress_fraction(guint64 const sent, guint64 const total)
 {
-  gdouble fraction;
+  draw_progress_args_t *dp;
   
-  fraction = ((gdouble) sent / (gdouble) total);
-  gdk_threads_enter();
-  gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(progress_bar), fraction);
-  gdk_threads_leave();
+  dp = (draw_progress_args_t *) g_malloc(sizeof(draw_progress_args_t));
+  dp->progress = progress_bar;
+  dp->fraction = ((gdouble) sent / (gdouble) total);
+  g_idle_add(draw_progress, dp);
+  // g_print("Sent: %llu, total: %llu, fraction: %f\n", sent, total, fraction);
 }
 
 /*
@@ -1805,17 +1729,34 @@ static int mtp_progress (uint64_t const sent, uint64_t const total,
 }
 #endif
 
-#ifdef G_OS_WIN32
-// Windows don't like it when other threads write to the GUI
-// so we do it here from the main loop instead.
-gboolean call_fill_in_dir(gpointer data)
+/*
+ * These functions only serves to send different drawing
+ * operations to the main loop.
+ */
+
+static gboolean call_fill_in_dir(gpointer data)
 {
   guint what = (guint) data;
 
   fill_in_dir(what, get_current_dir());
   return FALSE;
 }
-#endif
+
+typedef struct {
+  listtype_t listtype;
+  metadata_t *meta;
+} metadata_add_args_t;
+
+static gboolean call_add_metadata_to_model(gpointer data)
+{
+  metadata_add_args_t *args = (metadata_add_args_t *) data;
+
+  add_metadata_to_model(args->meta, args->listtype);
+  destroy_metadata_t(args->meta);
+  g_free(args);
+  return FALSE;
+}
+
 
 /***********************************************************************************/
 /* Transferring music from jukebox thread                                          */
@@ -1834,6 +1775,9 @@ gpointer jb2hd_thread(gpointer thread_args)
       u_int32_t id;
       gchar *filename;
       gchar *tmpfname;
+      draw_label_args_t *da;
+      draw_progress_args_t *dp;
+      metadata_add_args_t *metaddarg;
       
       /* Get title from filelist */
       jbmeta = (metadata_t *) tmplist->data;
@@ -1845,12 +1789,25 @@ gpointer jb2hd_thread(gpointer thread_args)
       // convert filenames into widechar unicode...
       tmpfname = g_locale_from_utf8(filename,-1,NULL,NULL,NULL);
 #endif
+      // Check if file exists to avoid overwriting.
+      // Add an underscore just before the extension if file exists.
+      while (g_file_test(tmpfname, G_FILE_TEST_EXISTS)) {
+	tmpfname = add_underscore(tmpfname);
+      }
+
       create_dirs(tmpfname);
       id = string_to_guint(jbmeta->path);
-      gdk_threads_enter();
-      gtk_label_set_text(GTK_LABEL(args->label), filename);
-      gtk_progress_bar_update(GTK_PROGRESS_BAR(progress_bar), (gfloat) 0);
-      gdk_threads_leave();
+
+      // Draw next file
+      da = (draw_label_args_t *) g_malloc(sizeof(draw_label_args_t));
+      da->label = args->label;
+      da->text = g_strdup(tmpfname);
+      g_idle_add(draw_label, da);
+      dp = (draw_progress_args_t *) g_malloc(sizeof(draw_progress_args_t));
+      dp->progress = progress_bar;
+      dp->fraction = (gfloat) 0;
+      g_idle_add(draw_progress, dp);
+
       /* g_print("Transferring %s...\n", file); */
       /* Transfer track */
       if (use_mtp) {
@@ -1874,7 +1831,7 @@ gpointer jb2hd_thread(gpointer thread_args)
       /* Copy all metadata, but replace jukebox ID with path */
       meta = clone_metadata_t(jbmeta);
       g_free(meta->path);
-      meta->path = g_strdup(filename);
+      meta->path = g_strdup(tmpfname);
       g_free(filename);
       /* Tag with ID3 unless already present */
       if (get_prefs_useid3()) {
@@ -1886,27 +1843,21 @@ gpointer jb2hd_thread(gpointer thread_args)
       
       /* Add to disk listbox and re-sort */
       g_free(tmpfname);
-      gdk_threads_enter();
-      add_metadata_to_model(meta, HD_LIST);
-      gdk_flush(); // Time consuming?
-      gdk_threads_leave();
-      /* Destroy metadata */
-      destroy_metadata_t(meta);
+
+      /* Send to main loop for drawing */
+      metaddarg = (metadata_add_args_t *) g_malloc(sizeof(metadata_add_args_t));
+      metaddarg->listtype = HD_LIST;
+      metaddarg->meta = meta;
+      g_idle_add(call_add_metadata_to_model,metaddarg);
+
       tmplist = tmplist->next;
     }
   
  clean_up_and_return:
 
   jukebox_locked = FALSE;
-#ifndef G_OS_WIN32
-  gdk_threads_enter();
-  gtk_widget_destroy(args->dialog);
-  fill_in_dir(HD_LIST, get_current_dir());
-  gdk_threads_leave();
-#else
-  gtk_widget_destroy(args->dialog);
+  g_idle_add((GSourceFunc) gtk_widget_destroy, args->dialog);
   g_idle_add(call_fill_in_dir, (gpointer) HD_LIST);
-#endif
   /* Free the memory used by the list */
   destroy_metalist(args->metalist);
   g_free(args);
@@ -1928,14 +1879,24 @@ gpointer jb2hd_data_thread(gpointer thread_args)
       metadata_t *jbmeta;
       u_int32_t id;
       gchar *tmpfname;
+      draw_label_args_t *da;
+      draw_progress_args_t *dp;
+      metadata_add_args_t *metaddarg;
       
       jbmeta = (metadata_t *) metalist->data;
       id = string_to_guint(jbmeta->path);
-      gdk_threads_enter();
-      gtk_label_set_text(GTK_LABEL(args->label), jbmeta->filename);
-      gtk_progress_bar_update(GTK_PROGRESS_BAR(progress_bar), (gfloat) 0);
-      gdk_threads_leave();
-      /* g_print("Transfering %s\n", jbmeta->filename); */
+
+      // Draw next file
+      da = (draw_label_args_t *) g_malloc(sizeof(draw_label_args_t));
+      da->label = args->label;
+      da->text = g_strdup(jbmeta->filename);
+      g_idle_add(draw_label, da);
+      dp = (draw_progress_args_t *) g_malloc(sizeof(draw_progress_args_t));
+      dp->progress = progress_bar;
+      dp->fraction = (gfloat) 0;
+      g_idle_add(draw_progress, dp);
+
+      /* g_print("Transferring %s\n", jbmeta->filename); */
       /* Transfer file */
 #ifndef G_OS_WIN32
       tmpfname = filename_fromutf8(jbmeta->filename);
@@ -1961,11 +1922,13 @@ gpointer jb2hd_data_thread(gpointer thread_args)
 	}
       }
       g_free(tmpfname);
-      /* Add to disk listbox and re-sort */
-      gdk_threads_enter();
-      add_metadata_to_model(jbmeta, HDDATA_LIST);
-      gdk_flush(); // Time consuming?
-      gdk_threads_leave();
+
+      /* Send to main loop for drawing */
+      metaddarg = (metadata_add_args_t *) g_malloc(sizeof(metadata_add_args_t));
+      metaddarg->listtype = HDDATA_LIST;
+      metaddarg->meta = jbmeta;
+      g_idle_add(call_add_metadata_to_model,metaddarg);
+
       metalist = g_list_next(metalist);
     }
   
@@ -1973,15 +1936,8 @@ gpointer jb2hd_data_thread(gpointer thread_args)
 
   destroy_metalist(args->metalist);
   jukebox_locked = FALSE;
-#ifndef G_OS_WIN32
-  gdk_threads_enter();
-  gtk_widget_destroy(args->dialog);
-  fill_in_dir(HDDATA_LIST, get_current_dir());
-  gdk_threads_leave();
-#else
-  gtk_widget_destroy(args->dialog);
+  g_idle_add((GSourceFunc) gtk_widget_destroy, args->dialog);
   g_idle_add(call_fill_in_dir, (gpointer) HDDATA_LIST);
-#endif
   g_free(args);
 }
 
@@ -2014,9 +1970,11 @@ gpointer hd2jb_thread(gpointer thread_args)
       gint protectint;
       gchar *tmpfname = NULL;
       gchar *tmpfname8 = NULL;
-      gchar *tmp;
       u_int32_t id, length;
       gboolean clone_and_strip = FALSE;
+      draw_label_args_t *da;
+      draw_progress_args_t *dp;
+      metadata_add_args_t *metaddarg;
 
       hdmeta = (metadata_t *) templist->data;
       // g_print("Storing %s on jukebox...\n", hdmeta->filename);
@@ -2033,13 +1991,17 @@ gpointer hd2jb_thread(gpointer thread_args)
       } else {
 	tmpfname = filename_fromutf8(hdmeta->path);
       }
+
+      // Draw next file
+      da = (draw_label_args_t *) g_malloc(sizeof(draw_label_args_t));
+      da->label = args->label;
+      da->text = g_strdup_printf("%s - %s", hdmeta->artist, hdmeta->title);
+      g_idle_add(draw_label, da);
+      dp = (draw_progress_args_t *) g_malloc(sizeof(draw_progress_args_t));
+      dp->progress = progress_bar;
+      dp->fraction = (gfloat) 0;
+      g_idle_add(draw_progress, dp);
       
-      gdk_threads_enter();
-      tmp = g_strdup_printf("%s - %s", hdmeta->artist, hdmeta->title);
-      gtk_label_set_text(GTK_LABEL(args->label), tmp);
-      g_free(tmp);
-      gtk_progress_bar_update(GTK_PROGRESS_BAR(progress_bar), (gfloat) 0);
-      gdk_threads_leave();
       length = mmss_to_seconds(hdmeta->length);
       protectint = hdmeta->protect ? 1 : 0;
 
@@ -2066,7 +2028,7 @@ gpointer hd2jb_thread(gpointer thread_args)
 	trackmeta->filesize = get_64bit_file_size(tmpfname);
 	trackmeta->filetype = filetype_d->filetype;
 	// TODO: Discarding return value as for now
-	g_print("Transferring MTP track...\n");
+	// g_print("Transferring MTP track...\n");
 	// Hardcode parent (folder) to 0 (== root directory)
 	ret = LIBMTP_Send_Track_From_File(mtpdevice, tmpfname, trackmeta,
 					  mtp_progress, NULL, 0);
@@ -2126,14 +2088,16 @@ gpointer hd2jb_thread(gpointer thread_args)
       jbmeta = clone_metadata_t(hdmeta);
       g_free(jbmeta->path);
       jbmeta->path = g_strdup_printf("%lu", id);
-      gdk_threads_enter();
-      add_metadata_to_model(jbmeta, JB_LIST);
-      gdk_flush();
-      gdk_threads_leave();
-      /* Then add the song to the global hash table */
+
+      /* Send to main loop for drawing */
+      metaddarg = (metadata_add_args_t *) g_malloc(sizeof(metadata_add_args_t));
+      metaddarg->listtype = JB_LIST;
+      metaddarg->meta = clone_metadata_t(jbmeta);
+      g_idle_add(call_add_metadata_to_model,metaddarg);
+
       g_hash_table_insert(songhash,
 			  GUINT_TO_POINTER(id),
-			  (gpointer) jbmeta);
+			  (gpointer) clone_metadata_t(jbmeta));
       new_metalist = g_list_append (new_metalist, jbmeta);
       if (clone_and_strip) {
 	/* Afterwards delete the tempfile */
@@ -2152,11 +2116,11 @@ gpointer hd2jb_thread(gpointer thread_args)
    * any such are selected.
    */
   if (args->playlists != NULL) {
-    add_tracks_to_playlists(args->playlists, new_metalist, args->pltreestore, TRUE);
+    add_tracks_to_playlists(args->playlists, new_metalist);
   }
-  g_list_free(new_metalist);
+  destroy_metalist(new_metalist);
   flush_usage();
-  
+ 
  hd2jb_cleanup:
 
   // Remove temporary directory for tag-stripped files.
@@ -2164,16 +2128,13 @@ gpointer hd2jb_thread(gpointer thread_args)
     rmdir(tmpdirname);
   }
   
+  g_idle_add((GSourceFunc) gtk_widget_destroy, args->dialog);
+  if (args->playlists != NULL) {
+    g_idle_add(call_build_playlist_tree, (gpointer) args->pltreestore);
+  } 
   jukebox_locked = FALSE;
   g_list_free(args->playlists);
   destroy_metalist(args->metalist);
-#ifndef G_OS_WIN32
-  gdk_threads_enter();
-  gtk_widget_destroy(args->dialog);
-  gdk_threads_leave();
-#else
-  gtk_widget_destroy(args->dialog);
-#endif
   g_free(args);
 }
 
@@ -2196,18 +2157,29 @@ gpointer hd2jb_data_thread(gpointer thread_args)
       u_int32_t id;
       metadata_t *hdmeta;
       metadata_t *jbmeta;
+      draw_label_args_t *da;
+      draw_progress_args_t *dp;
+      metadata_add_args_t *metaddarg;
       
       hdmeta = (metadata_t *) metalist->data;
+      // FIXME: is this getting operation really thread safe???
       folder = (gchar *) gtk_entry_get_text(GTK_ENTRY(data_widgets.jbentry));
       if (folder == NULL) {
 	/* This shouldn't happen, but include it anyway. */
 	folder = rootdir;
       }
-      g_print("Storing %s (extension %s) on jukebox as %s in folder %s...\n", hdmeta->path, hdmeta->codec, hdmeta->filename, folder);
-      gdk_threads_enter();
-      gtk_label_set_text(GTK_LABEL(args->label), hdmeta->filename);
-      gtk_progress_bar_update(GTK_PROGRESS_BAR(progress_bar), (gfloat) 0);
-      gdk_threads_leave();
+
+      // Draw next file
+      // g_print("Storing %s (extension %s) on jukebox as %s in folder %s...\n", hdmeta->path, hdmeta->codec, hdmeta->filename, folder);
+      da = (draw_label_args_t *) g_malloc(sizeof(draw_label_args_t));
+      da->label = args->label;
+      da->text = g_strdup(hdmeta->filename);
+      g_idle_add(draw_label, da);
+      dp = (draw_progress_args_t *) g_malloc(sizeof(draw_progress_args_t));
+      dp->progress = progress_bar;
+      dp->fraction = (gfloat) 0;
+      g_idle_add(draw_progress, dp);
+
       tmpfname = filename_fromutf8(hdmeta->path);
       if (use_mtp) {
 #ifdef HAVE_LIBMTP
@@ -2248,9 +2220,13 @@ gpointer hd2jb_data_thread(gpointer thread_args)
 	g_free(jbmeta->folder);
       }
       jbmeta->folder = g_strdup(folder);
-      gdk_threads_enter();
-      add_metadata_to_model(jbmeta, JBDATA_LIST);
-      gdk_threads_leave();
+
+      /* Send to main loop for drawing */
+      metaddarg = (metadata_add_args_t *) g_malloc(sizeof(metadata_add_args_t));
+      metaddarg->listtype = JBDATA_LIST;
+      metaddarg->meta = jbmeta;
+      g_idle_add(call_add_metadata_to_model,metaddarg);
+
       /* Keep this in the global metadata file list! */
       datafilelist = g_slist_append(datafilelist, (gpointer) jbmeta);
       metalist = g_list_next(metalist);
@@ -2259,15 +2235,9 @@ gpointer hd2jb_data_thread(gpointer thread_args)
 
  hd2jb_data_cleanup:
 
+  g_idle_add((GSourceFunc) gtk_widget_destroy, args->dialog);
   jukebox_locked = FALSE;
   destroy_metalist(args->metalist);
-#ifndef G_OS_WIN32
-  gdk_threads_enter();
-  gtk_widget_destroy(args->dialog);
-  gdk_threads_leave();
-#else
-  gtk_widget_destroy(args->dialog);
-#endif
   g_free(args);
 }
 
@@ -2376,7 +2346,7 @@ static void remove_tracks_from_playlists(GList *metalist,
     }
     tmplist = g_list_next(tmplist);
   }
-  build_playlist_tree(pltreestore, FALSE);
+  build_playlist_tree(pltreestore);
 }
 
 /* Delete a list of files from the jukebox */
@@ -2599,7 +2569,7 @@ void jukebox_end_metadata_set(void)
   /* End metadata transaction */
   jukebox_locked = FALSE;
   /* Update the window with the new metadata */
-  jblist_from_songhash(FALSE);
+  jblist_from_songhash();
 }
 
 /* Creates a new playlist and returns its playlist ID UNTHREADED */
@@ -2642,7 +2612,7 @@ guint jukebox_create_playlist(gchar *plname, GtkTreeStore *pltreestore)
     plid = playlist->plid;
     NJB_Playlist_Destroy(playlist);
   }
-  build_playlist_tree(pltreestore, FALSE);
+  build_playlist_tree(pltreestore);
   jukebox_locked = FALSE;
   /* g_print("Created playlist\n"); */
   return plid;
@@ -2669,8 +2639,8 @@ void jukebox_delete_playlist(guint plid)
   jukebox_locked = FALSE;
 }
 
-/* Rename the playlist with ID plid UNTHREADED */
-void jukebox_rename_playlist(guint plid, gchar *name, GtkTreeStore *pltreestore, gboolean threaded)
+/* Rename the playlist with ID plid */
+void jukebox_rename_playlist(guint plid, gchar *name, GtkTreeStore *pltreestore)
 {
   if (use_mtp) {
 #ifdef HAVE_LIBMTP
@@ -2683,11 +2653,7 @@ void jukebox_rename_playlist(guint plid, gchar *name, GtkTreeStore *pltreestore,
     }
     pl->name = g_strdup(name);
     if (LIBMTP_Update_Playlist(mtpdevice, pl) != 0) {
-      if (threaded)
-	gdk_threads_enter();
       create_error_dialog(_("Could not rename playlist"));
-      if (threaded)
-	gdk_threads_leave();
     }
     LIBMTP_destroy_playlist_t(pl);
 #endif
@@ -2712,45 +2678,141 @@ void jukebox_rename_playlist(guint plid, gchar *name, GtkTreeStore *pltreestore,
 	NJB_Error_Dump(pdedevice, stderr);
       } else {
 	if (NJB_Update_Playlist(pdedevice, playlist) == -1) {
-	  if (threaded)
-	    gdk_threads_enter();
 	  create_error_dialog(_("Could not rename playlist"));
-	  if (threaded)
-	    gdk_threads_leave();
 	  NJB_Error_Dump(pdedevice, stderr);
 	}
       }
       NJB_Playlist_Destroy(playlist);
-      build_playlist_tree(pltreestore, FALSE);
+      build_playlist_tree(pltreestore);
     } else {
-      if (threaded)
-	gdk_threads_enter();
       create_error_dialog(_("Could not locate playlist to rename!"));
-      if (threaded)
-	gdk_threads_leave();
     }
     jukebox_locked = FALSE;
   }
 }
 
-/* This is a public caller func for the above UNTHREADED */
-void add_tracks_to_playlists(GList *playlists, GList *metalist, GtkTreeStore *pltreestore, gboolean threaded)
+static void add_tracks_to_single_playlist(guint playlistid, GList *metalist)
+{
+  // Sanity check
+  if (!metalist)
+    return;
+
+  if (use_mtp) {
+#ifdef HAVE_LIBMTP
+    LIBMTP_playlist_t *pl;
+    GList *tmplist;
+    guint i;
+    guint toadd;
+    uint32_t *copy;
+
+    if (gnomad_debug != 0) {
+      g_print("Getting playlist %d\n", playlistid);
+    }
+    pl = LIBMTP_Get_Playlist(mtpdevice, playlistid);
+    if (pl == NULL) {
+      return;
+    }
+
+    // Count the tracks and allocate space for playlist
+    tmplist = metalist;
+    toadd = 0;
+    while(tmplist) {
+      toadd++;
+      tmplist = g_list_next(tmplist);
+    }
+    
+    // Allocate the new playlist
+    copy = g_malloc((pl->no_tracks + toadd) * sizeof(uint32_t));
+    
+    // Copy old tracks
+    for (i = 0; i < pl->no_tracks; i++) {
+      copy[i] = pl->tracks[i];
+    }
+
+    // Remove old contents
+    if (pl->tracks != NULL) {
+      g_free(pl->tracks);
+    }
+    pl->tracks = copy;
+    pl->no_tracks = pl->no_tracks + toadd;
+
+    // Add each new track
+    tmplist = metalist;
+    while(tmplist) {
+      metadata_t *meta;
+      uint32_t id;
+      
+      meta = (metadata_t *) tmplist->data;
+      id = string_to_guint(meta->path);
+      if (gnomad_debug != 0) {
+	g_print("Adding track %lu to playlist %lu\n", id, playlistid);
+      }
+      pl->tracks[i] = id;
+      tmplist = g_list_next(tmplist);
+      i++;
+    }
+    if (LIBMTP_Update_Playlist(mtpdevice, pl) != 0) {
+      g_print("Failed to add tracks to playlist on MTP device.\n");
+    }
+#endif
+  } else {
+    njb_playlist_track_t *pl_track;
+    njb_playlist_t *playlist;
+    GList *tmplist = metalist;
+    gboolean found = FALSE;
+    
+    NJB_Reset_Get_Playlist(pdedevice);
+    while (playlist = NJB_Get_Playlist(pdedevice)) {
+      /* g_print("Is it %lu?\n", playlist->plid); */
+      if (playlist->plid == playlistid) {
+	/* g_print("YES!\n"); */
+	found = TRUE;
+	break;
+      }
+      /* g_print("NO.\n"); */
+      NJB_Playlist_Destroy(playlist);
+    }
+    if (!found) {
+      g_print(_("Could not find playlist"));
+    } else if (tmplist) {
+      while(tmplist) {
+	metadata_t *meta;
+	u_int32_t id;
+	
+	meta = (metadata_t *) tmplist->data;
+	id = string_to_guint(meta->path);
+	/* g_print("Adding track %lu to playlist %lu\n", id, playlistid); */
+	pl_track = NJB_Playlist_Track_New(id);
+	NJB_Playlist_Addtrack(playlist, pl_track, NJB_PL_END);
+	tmplist = g_list_next(tmplist);
+      }
+      if (NJB_Update_Playlist(pdedevice, playlist) == -1)
+	NJB_Error_Dump(pdedevice, stderr);
+      NJB_Playlist_Destroy(playlist);
+    }
+  }
+}
+
+/**
+ * Add a list of tracks to a list of playlists.
+ */
+void add_tracks_to_playlists(GList *playlists, GList *metalist)
 {
   GList *templist = g_list_first(playlists);
 
-  /* g_print("Called add_tracks_to_playlist()\n"); */
+  if (gnomad_debug != 0) {
+    g_print("Called add_tracks_to_playlist()\n");
+  }
   jukebox_locked = TRUE;
-  /* For thread message in the unthreaded code. Kludgy. */
   cancel_jukebox_operation = FALSE;
   while (templist != NULL) {
     guint plid = GPOINTER_TO_UINT(templist->data);
-    add_to_playlist(plid, metalist, pltreestore, threaded);
+    add_tracks_to_single_playlist(plid, metalist);
     templist = g_list_next(templist);
   }
   jukebox_locked = FALSE;
 }
 
-
 /* Randomize file from playlist UNTHREADED */
 guint jukebox_randomize_playlist(guint plist, GtkTreeStore *pltreestore)
 {
@@ -2837,7 +2899,7 @@ guint jukebox_randomize_playlist(guint plist, GtkTreeStore *pltreestore)
       g_ptr_array_free(contents, FALSE);
       g_rand_free(randnum);
       // Then rebuild the playlist list (necessary).
-      build_playlist_tree(pltreestore, FALSE);
+      build_playlist_tree(pltreestore);
     }
   }
   jukebox_locked = FALSE;
@@ -2921,7 +2983,7 @@ guint jukebox_delete_track_from_playlist(guint trackid, guint plist, GtkTreeStor
   jukebox_locked = FALSE;
   // g_print("Old playlist ID: %lu, new playlist ID: %lu\n", plist, new_plid);
   // When replacing playlist ID does not work we must use this
-  // build_playlist_tree(pltreestore, FALSE);
+  // build_playlist_tree(pltreestore);
   return new_plid;
 }
 
@@ -3108,15 +3170,16 @@ static void send_playlist(GList *list)
 }
 
 /* Sets the songname in the dialog - you have to
- * lock mutexes and gdk thread (if needed) before 
+ * lock mutexes before 
  * calling this function */
 static void set_songname_label(gchar *artist, gchar *title, gchar *seconds)
 {
-  gchar *tmp;
+  draw_label_args_t *da;
 
-  tmp = g_strdup_printf("%s - %s, %s", artist, title, seconds);
-  gtk_label_set_text(GTK_LABEL(songnamelabel), tmp);
-  g_free(tmp);
+  da = (draw_label_args_t *) g_malloc(sizeof(draw_label_args_t));
+  da->label = songnamelabel;
+  da->text = g_strdup_printf("%s - %s, %s", artist, title, seconds);
+  g_idle_add(draw_label, da);
 }
 
 /**
@@ -3206,6 +3269,20 @@ static void jukebox_current(void)
   }
 }
 
+typedef struct {
+  GtkAdjustment *adjustment;
+  gdouble position;
+} set_adjustment_args_t;
+
+static gboolean call_set_adjustment_value(gpointer data)
+{
+  set_adjustment_args_t *sa = (set_adjustment_args_t *) data;
+  
+  gtk_adjustment_set_value(sa->adjustment, sa->position);
+  g_free(sa);
+  return FALSE;
+}
+
 /* Start playing a bunch of selected files */
 gpointer play_thread(gpointer thread_args)
 {
@@ -3241,9 +3318,8 @@ gpointer play_thread(gpointer thread_args)
   /* Set playlistlast to the pointer of the last song */
   playlistlast = g_list_last(playlist);
   /* Setup for the first round */
-  gdk_threads_enter();
   jukebox_current();
-  gdk_threads_leave();
+
   g_mutex_unlock(play_thread_mutex);
 
   while (!cancel_jukebox_operation &&
@@ -3256,15 +3332,15 @@ gpointer play_thread(gpointer thread_args)
 	 * is not executed on any device I've seen. */
 	repeat = FALSE;
       } else {
-	gdk_threads_enter();
 	jukebox_next(TRUE);
-	gdk_threads_leave();
       }
       change = 0;
     } else {
       gchar tmp[10];
       metadata_t *meta;
       guint seconds;
+      draw_label_args_t *da;
+      set_adjustment_args_t *sa;
 
       g_mutex_lock(play_thread_mutex);
       NJB_Elapsed_Time(pdedevice, &sec, &change);
@@ -3280,10 +3356,16 @@ gpointer play_thread(gpointer thread_args)
        * tag may be incorrect */
       if (position < 0)
 	position = 100.0;
-      gdk_threads_enter();
-      gtk_label_set_text(GTK_LABEL(args->timelabel), tmp);
-      gtk_adjustment_set_value(GTK_ADJUSTMENT(args->adj), position);
-      gdk_threads_leave();
+      
+      da = (draw_label_args_t *) g_malloc(sizeof(draw_label_args_t));
+      da->label = args->timelabel;
+      da->text = g_strdup(tmp);
+      g_idle_add(draw_label, da);
+
+      sa = (set_adjustment_args_t *) g_malloc(sizeof(set_adjustment_args_t));
+      sa->adjustment = args->adj;
+      sa->position = position;
+      g_idle_add(call_set_adjustment_value, sa);
     }
     /* The Jukebox reports zero seconds of time at the
      * beginning of play, and after the song ends. This
@@ -3305,17 +3387,10 @@ gpointer play_thread(gpointer thread_args)
     sleep(1);
   }
 
-  /* Free the argument list */
-
-#ifndef G_OS_WIN32
-  gdk_threads_enter();
-  gtk_widget_destroy(args->dialog);
-  gdk_threads_leave();
-#else
-  // Let the main loop destroy the dialog on Win32
-  g_idle_add(gtk_widget_destroy, (gpointer) args->dialog);
-#endif
+  // Let the main loop destroy the dialog
+  g_idle_add((GSourceFunc) gtk_widget_destroy, args->dialog);
   g_mutex_lock(play_thread_mutex);
+  // Free the playlist
   destroy_metalist(playlist);
   playlist = NULL;
   NJB_Stop_Play(pdedevice);
