@@ -1,7 +1,7 @@
 /* jukebox.h
    Functions related to the communication with jukebox hardware
    using the libnjb library, header file
-   Copyright (C) 2001 Linus Walleij
+   Copyright (C) 2001-2007 Linus Walleij
 
 This file is part of the GNOMAD package.
 
@@ -77,7 +77,7 @@ typedef struct
   GtkWidget *dialog;
   GtkWidget *songlabel;
   GtkWidget *timelabel;
-  GtkObject *adj;
+  GtkAdjustment *adj;
   GList *metalist;
 } play_thread_arg_t;
 
@@ -110,10 +110,9 @@ void jukebox_end_metadata_set(void);
 guint jukebox_create_playlist(gchar *plname, GtkTreeStore *pltreestore);
 void jukebox_delete_playlist(guint plid);
 void jukebox_rename_playlist(guint plid, gchar *name, 
-			     GtkTreeStore *pltreestore,
-			     gboolean threaded);
-void add_tracks_to_playlists(GList *playlists, GList *metalist, 
-			     GtkTreeStore *pltreestore, gboolean threaded);
+			     GtkTreeStore *pltreestore);
+void add_tracks_to_playlists(GList *playlists, GList *metalist);
+void build_playlist_tree(GtkTreeStore *pltreestore);
 guint jukebox_randomize_playlist(guint playlist, GtkTreeStore *pltreestore);
 guint jukebox_delete_track_from_playlist(guint trackid, guint plist, GtkTreeStore *pltreestore);
 gchar *jukebox_get_ownerstring(void);
