@@ -245,6 +245,7 @@ metadata_t *get_data_metadata_from_model(GtkTreeModel *model,
 		     FILELIST_SIZE_COLUMN, &meta->size,
 		     FILELIST_ID_COLUMN, &meta->path,
 		     FILELIST_FOLDER_COLUMN, &meta->folder,
+		     FILELIST_EXTENSION_COLUMN, &meta->codec,
 		     -1);
   return meta;
 }
@@ -292,6 +293,7 @@ void add_metadata_to_model(metadata_t const * const meta,
 			FILELIST_SIZE_COLUMN, meta->size,
 			FILELIST_ID_COLUMN, meta->path,
 			FILELIST_FOLDER_COLUMN, meta->folder,
+			FILELIST_EXTENSION_COLUMN, meta->codec,
 			-1);
   }
 }
@@ -609,7 +611,8 @@ static GtkListStore *create_hddataliststore()
 					G_TYPE_STRING, /*Filename */
 					G_TYPE_INT, /* Size */
 					G_TYPE_STRING, /*ID*/
-					G_TYPE_STRING /* Folder */
+					G_TYPE_STRING, /* Folder */
+					G_TYPE_STRING /* Codec */
 				    );
   return hddataliststore;
 }
@@ -622,7 +625,8 @@ static GtkListStore *create_jbdataliststore()
 					G_TYPE_STRING, /*Filename */
 					G_TYPE_INT, /* Size */
 					G_TYPE_STRING, /*ID*/
-					G_TYPE_STRING /* Folder */
+					G_TYPE_STRING, /* Folder */
+					G_TYPE_STRING /* Codec */
 				    );
   return jbdataliststore;
 }
