@@ -19,6 +19,7 @@ Boston, MA 02111-1307, USA.
 #include <sys/stat.h>
 #include "common.h"
 #include "util.h"
+#include "filesystem.h"
 
 /*
  * The following code is extracted from gtkpod, which in turn
@@ -200,6 +201,9 @@ void get_mp3_info(mp3info *mp3)
   struct stat filestat;
   off_t data_start=0;
 
+  if (gnomad_debug != 0) {
+    g_print("entered get_mp3_info()\n");
+  }
 
   stat(mp3->filename,&filestat);
   mp3->datasize=filestat.st_size;
@@ -279,7 +283,7 @@ gint get_next_header(mp3info *mp3)
   mp3header h;
 
    while(1) {
-     while((c=fgetc(mp3->file)) != 255 && (ftell(mp3->file) < mp3->datasize)) skip_bytes++;
+     while((c=fgetc(mp3->file)) != 255 && !feof(mp3->file) && (ftell(mp3->file) < mp3->datasize)) skip_bytes++;
      if(c == 255) {
 	ungetc(c,mp3->file);
 	if((l=get_header(mp3->file,&h))) {
@@ -372,9 +376,17 @@ static mp3info *mp3file_get_info (gchar *filename)
 {
   mp3info *mp3 = NULL;
   FILE *fp;
+
+  if (gnomad_debug != 0) {
+    g_print("entered mp3file_get_info()\n");
+  }
   
-  if ( !( fp=fopen(filename,"r") ) ) {
-    g_print(("Error opening MP3 file '%s'"),filename);
+#if !GTK_CHECK_VERSION(2,6,0)  
+  if ( !( fp = fopen(filename,READONLY_FOPEN_STRING) ) ) {
+#else
+  if ( !( fp = g_fopen(filename,READONLY_FOPEN_STRING) ) ) {
+#endif
+    g_print(("Error opening MP3 file '%s'\n"),filename);
   } else {
     mp3 = g_malloc0 (sizeof (mp3info));
     mp3->filename=filename;
@@ -913,7 +925,11 @@ static guint get_track_time (gchar *path)
   guint result = 0;
   
   if (path) {
-    FILE *file = fopen (path, "r");
+#if !GTK_CHECK_VERSION(2,6,0)  
+    FILE *file = fopen (path, READONLY_FOPEN_STRING);
+#else
+    FILE *file = g_fopen (path, READONLY_FOPEN_STRING);
+#endif
     result = get_track_time_file (file);
     if (file) fclose (file);
   }
