@@ -436,7 +436,7 @@ void create_player_window(GList *metalist)
   GtkWidget *label, *arrow;
   GtkWidget *hscale, *eax_controls;
   GtkWidget *hbox, *hbox2, *hbox3, *vbox;
-  GtkObject *adj;
+  GtkAdjustment *adj;
 
   // Playback on MTP devices not supported. (Yet.)
   if (jukebox_is_mtp()) {
@@ -456,8 +456,8 @@ void create_player_window(GList *metalist)
   /* Note that the page_size value only makes a difference for
    * scrollbar widgets, and the highest value you'll get is actually
    * (upper - page_size). */
-  adj = gtk_adjustment_new (0.0, 0.0, 101.0, 0.1, 1.0, 1.0);
-  hscale = gtk_hscale_new (GTK_ADJUSTMENT (adj));
+  adj = GTK_ADJUSTMENT(gtk_adjustment_new(0.0, 0.0, 101.0, 0.1, 1.0, 1.0));
+  hscale = gtk_hscale_new(adj);
   // To make it possible to skip into the song
   // gtk_widget_set_sensitive(hscale, FALSE);
   g_signal_connect_data(GTK_OBJECT(adj),
