@@ -245,7 +245,7 @@ static GCallback edit_playlist_ok_button (GtkButton *button,
 
   plid = get_playlistid_from_selection();
   plname = g_strdup(gtk_entry_get_text(GTK_ENTRY(edit_pl_entry)));
-  jukebox_rename_playlist(plid, plname, playlist_widgets.pltreestore, FALSE);
+  jukebox_rename_playlist(plid, plname, playlist_widgets.pltreestore);
   gtk_widget_destroy(edit_pl_dialog);
 
   g_free(plname);
@@ -571,12 +571,14 @@ static GCallback playlist_delete_response(gpointer data)
     create_error_dialog(_("Jukebox busy"));
     return;
   }
-  if (request_confirmation_dialog(_("Really delete selected playlist?"))) {
-    plid = get_playlistid_from_selection();
-    jukebox_delete_playlist(plid);
-    /* g_print("Deleting playlist %lu...\n", plid); */
-    remove_selected_row();
-  }
+  plid = get_playlistid_from_selection();
+  if (plid) {
+      if (request_confirmation_dialog(_("Really delete selected playlist?"))) {
+	jukebox_delete_playlist(plid);
+	/* g_print("Deleting playlist %lu...\n", plid); */
+	remove_selected_row();
+      }
+    }
 }
 
 /* Response to the Shuffle playlist popup menu function */
@@ -829,7 +831,8 @@ static void add_to_playlist_ok_button (GtkButton *button,
     return;
   }
   /* g_print("Calling add_tracks_to_playlists()\n"); */
-  add_tracks_to_playlists(listlist, metalist, playlist_widgets.pltreestore, FALSE);
+  add_tracks_to_playlists(listlist, metalist);
+  build_playlist_tree(playlist_widgets.pltreestore);
   /* Free any used memory */
   g_list_free(listlist);
   g_list_free(selected_target_playlists);
