@@ -281,13 +281,33 @@ gboolean request_confirmation_dialog(gchar *confirmstring)
 
 void hexdump(unsigned char *data, guint len)
 {
-  guint i;
+  guint i,j,k;
 
   for (i=0; i<len; i++) {
-    if (i%16 == 0 && i != 0)
+    if (i % 16 == 0 && i != 0) {
+      g_print("  ");
+      
+      for (j = i-16; j < i; j++) {
+	gchar ch = data[j];
+	g_print("%c", ( ch >= 0x20 && ch <= 0x7e ) ? ch : '.');
+      }
       g_print("\n");
+    }
     g_print("%02x ", (guchar) data[i]);
   }
-  g_print("\n");
+  k = i % 16;
+  if (k != 0) {
+    for (j = 0; j < k+2; j++) {
+      g_print("   ");
+    }
+  } else {
+    k = 16;
+  }
+  g_print("  ");
+  for (j = i-k; j < i; j++) {
+    gchar ch = data[j];
+    g_print("%c", ( ch >= 0x20 && ch <= 0x7e ) ? ch : '.');
+  }
+  g_print("\n\n");
 }
 
