@@ -34,6 +34,10 @@ Boston, MA 02111-1307, USA.
 #define GNOME_DIALOG GTK_DIALOG
 #endif
 
+/* Local variables */
+static GtkWidget *new_folder_dialog;
+static GtkWidget *new_folder_entry;
+
 /*****************************************************************************
  * Here comes the jukebox transfer functions.
  *****************************************************************************/
@@ -57,7 +61,7 @@ static void transfer_from_jukebox_to_hd (GtkButton *button,
     GtkWidget *label1, *label2, *dialog, *button, *separator;
 
 #if !GTK_CHECK_VERSION(2,6,0)    
-    dialog = gnome_dialog_new(_("Transfering tracks from jukebox library"),
+    dialog = gnome_dialog_new(_("Transferring tracks from jukebox library"),
 			      GNOME_STOCK_BUTTON_CANCEL,
 			      NULL);
     gnome_dialog_button_connect(GNOME_DIALOG(dialog),
@@ -65,7 +69,7 @@ static void transfer_from_jukebox_to_hd (GtkButton *button,
 				G_CALLBACK(cancel_jukebox_operation_click),
 				NULL);
 #else
-    dialog=gtk_dialog_new_with_buttons(_("Transfering tracks from jukebox library"),
+    dialog=gtk_dialog_new_with_buttons(_("Transferring tracks from jukebox library"),
 		    NULL,
 		    0,
 		    NULL);
@@ -158,7 +162,7 @@ void transfer_from_hd_to_jukebox(GList *playlists)
 
     hd2jb_thread_args = (hd2jb_thread_arg_t *) malloc(sizeof (hd2jb_thread_arg_t));
 #if !GTK_CHECK_VERSION(2,6,0)
-    dialog = gnome_dialog_new(_("Transfering tracks to jukebox library"),
+    dialog = gnome_dialog_new(_("Transferring tracks to jukebox library"),
 			      GNOME_STOCK_BUTTON_CANCEL,
 			      NULL);
     gnome_dialog_button_connect(GNOME_DIALOG(dialog),
@@ -166,7 +170,7 @@ void transfer_from_hd_to_jukebox(GList *playlists)
 				G_CALLBACK(cancel_jukebox_operation_click),
 				NULL);
 #else
-     dialog=gtk_dialog_new_with_buttons(_("Transfering tracks to jukebox library"),
+     dialog=gtk_dialog_new_with_buttons(_("Transferring tracks to jukebox library"),
 		    NULL,
 		    0,
 		    NULL);
@@ -183,7 +187,7 @@ void transfer_from_hd_to_jukebox(GList *playlists)
 			    G_CALLBACK(gtk_widget_destroy),
 			    GTK_OBJECT(dialog),
 			    0);
-    gtk_window_set_title (GTK_WINDOW (dialog), _("Transfering tracks to jukebox library"));
+    gtk_window_set_title (GTK_WINDOW (dialog), _("Transferring tracks to jukebox library"));
     gtk_window_set_position (GTK_WINDOW (dialog), GTK_WIN_POS_MOUSE);
     label1 = gtk_label_new(_("Storing tracks in Jukebox library..."));
     label2 = gtk_label_new("");   /* FIXME: Instead add a list to the dialog so progress may be seen better */
@@ -269,15 +273,96 @@ static GCallback hdmenu_transfer_response(gpointer data)
 /* Response to the "delete" call from the harddisk popup menu */
 static GCallback hdmenu_delete_response(gpointer data)
 {
-  if (request_confirmation_dialog(_("Really delete selected tracks?"))) {
+  if (request_confirmation_dialog(_("Really delete selected files/folders?"))) {
     GList *metalist = get_all_metadata_from_selection(HD_LIST);
 
-    delete_files(metalist);
-    remove_selected(HD_LIST); // This or fill in directory? This is faster.
+    if (!delete_files(metalist)) {
+      remove_selected(HD_LIST); // This or fill in directory? This is faster.
+    }
+    else fill_in_dir(HD_LIST, get_current_dir());
     destroy_metalist(metalist);
   }
 }
 
+static GCallback create_new_folder_ok_button (GtkButton *button,
+					      gpointer data)
+{
+  gchar *foldername;
+  gchar *current;
+  gchar *fullpath;
+  current = get_current_dir();
+  foldername = g_strdup(gtk_entry_get_text(GTK_ENTRY(new_folder_entry)));
+  gtk_widget_destroy(new_folder_dialog);
+
+  fullpath = g_strconcat(current,"/",foldername,NULL);
+
+  if(g_mkdir_with_parents(fullpath,0755)) create_error_dialog(_("Could not create folder"));
+
+  fill_in_dir(HD_LIST, get_current_dir());
+
+  g_free(fullpath);
+  g_free(current);
+  g_free(foldername);
+}
+
+/* Response to the "new folder..." call from the harddisk popup menu */
+static GCallback hdmenu_new_folder_response(gpointer data)
+{
+  GtkWidget *label, *button;
+#if !GTK_CHECK_VERSION(2,6,0)  
+  new_folder_dialog = gnome_dialog_new(_("New folder"),
+				   GNOME_STOCK_BUTTON_CANCEL,
+				   GNOME_STOCK_BUTTON_OK,
+				   NULL);
+  gnome_dialog_button_connect_object(GNOME_DIALOG(new_folder_dialog),
+				     0,
+				     GTK_SIGNAL_FUNC(gtk_widget_destroy),
+				     GTK_OBJECT(new_folder_dialog));
+  gnome_dialog_button_connect(GNOME_DIALOG(new_folder_dialog),
+			      1,
+			      GTK_SIGNAL_FUNC(create_new_folder_ok_button),
+			      NULL);
+#else
+  new_folder_dialog=gtk_dialog_new_with_buttons(_("New folder"),
+					    NULL, 
+					    0, 
+					    NULL);
+
+  button = gtk_dialog_add_button(GTK_DIALOG(new_folder_dialog), 
+				 GTK_STOCK_CANCEL, 
+				 0);
+  g_signal_connect(button, 
+		   "clicked", 
+		   GTK_SIGNAL_FUNC(dispose_of_dialog_window), 
+		   GTK_OBJECT(new_folder_dialog));
+
+  button = gtk_dialog_add_button(GTK_DIALOG(new_folder_dialog), 
+				 GTK_STOCK_OK, 
+				 1);
+  g_signal_connect(button, 
+		   "clicked", 
+		   GTK_SIGNAL_FUNC(create_new_folder_ok_button), 
+		   NULL);
+#endif
+        
+  
+  gtk_window_set_position (GTK_WINDOW(new_folder_dialog), GTK_WIN_POS_MOUSE);
+
+  label = gtk_label_new(_("Enter a name for the folder:"));
+  gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(new_folder_dialog)->vbox), label, TRUE, TRUE, 0);
+  gtk_widget_show(label);
+  new_folder_entry = gtk_entry_new();
+  gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(new_folder_dialog)->vbox), new_folder_entry, TRUE, TRUE, 0);
+  gtk_widget_show(new_folder_entry);
+
+  g_signal_connect(G_OBJECT (new_folder_dialog), 
+		   "delete_event",
+		   G_CALLBACK(gtk_widget_destroy),
+		   G_OBJECT (new_folder_dialog));
+
+  gtk_widget_show_all(new_folder_dialog);
+}
+
 /* Handles descending into a directory in the clist */
 static void go_down()
 {
@@ -696,6 +781,14 @@ GtkWidget *create_xfer_widgets()
 			  NULL,
 			  0);
   gtk_widget_show (popupmenu_item);
+  popupmenu_item  = gtk_menu_item_new_with_label (_("New folder..."));
+  gtk_menu_shell_append (GTK_MENU_SHELL(popupmenu), popupmenu_item);
+  g_signal_connect_object(GTK_OBJECT(popupmenu_item),
+			  "activate",
+			  G_CALLBACK(hdmenu_new_folder_response),
+			  NULL,
+			  0);
+  gtk_widget_show (popupmenu_item);
   transfer_widgets.harddiskpopupmenu = popupmenu;
 
   /* The popup menu for the jukebox files */
